@@ -1,16 +1,13 @@
 #define _XOPEN_SOURCE 700
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+
 #include "cache.h"
 /*
 
 cache.c
 
- Related to cache whcih contains paths to cached files.
+ Related to cache which contains paths to cached files.
  
 */
-Cache *tempCache = NULL;
 
 Cache *createCache()
 {
@@ -43,16 +40,6 @@ void deleteCache(Cache *cache)
         }
 }
 
-void deleteCachedFiles(Cache *cache)
-{
-        CacheNode *current = cache->head;
-        while (current != NULL)
-        {
-                remove(current->filePath);
-                current = current->next;
-        }
-}
-
 bool existsInCache(Cache *cache, char *filePath)
 {
         CacheNode *current = cache->head;
@@ -65,4 +52,4 @@ bool existsInCache(Cache *cache, char *filePath)
                 current = current->next;
         } 
         return false;       
-}
\ No newline at end of file
+}
