@@ -1,28 +1,36 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdbool.h>
-#include <string.h>
 #include <ctype.h>
 #include <dirent.h>
-#include <sys/types.h>
 #include <regex.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/types.h>
 #include "file.h"
-#include "stringfunc.h"
-#include "albumart.h"
+#include "utils.h"
 
 #ifndef FILE_SYSTEM_ENTRY
 #define FILE_SYSTEM_ENTRY
 typedef struct FileSystemEntry
 {
-        char name[256];
-        char fullPath[PATH_MAX]; 
+        int id;
+        char *name;
+        char *fullPath;
         int isDirectory; // 1 for directory, 0 for file
         int isEnqueued;
+        int parentId;
         struct FileSystemEntry *parent;
         struct FileSystemEntry *children;
-        struct FileSystemEntry *next; // For siblings (next node in the same directory)        
+        struct FileSystemEntry *next; // For siblings (next node in the same directory)
 } FileSystemEntry;
 #endif
 
+#ifndef SLOWLOADING_CALLBACK
+#define SLOWLOADING_CALLBACK
+typedef void (*SlowloadingCallback)(void);
+#endif
+
 FileSystemEntry *createDirectoryTree(const char *startPath, int *numEntries);
 void freeTree(FileSystemEntry *root);
+void freeAndWriteTree(FileSystemEntry *root, const char *filename);
+FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *startMusicPath, int *numDirectoryEntries);
