@@ -1,5 +1,3 @@
-#include <stdbool.h>
-
 enum EventType
 {
         EVENT_NONE,
@@ -16,6 +14,7 @@ enum EventType
         EVENT_ADDTOMAINPLAYLIST,
         EVENT_DELETEFROMMAINPLAYLIST,
         EVENT_EXPORTPLAYLIST,
+        EVENT_UPDATELIBRARY,
         EVENT_SHUFFLE,
         EVENT_KEY_PRESS,
         EVENT_SHOWKEYBINDINGS,
