@@ -33,55 +33,51 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. */
 #define __USE_POSIX
 #endif
 
+#include <dirent.h>
+#include <fcntl.h>
+#include <gio/gio.h>
+#include <glib.h>
+#include <glib-unix.h>
+#include <poll.h>
+#include <pwd.h>
+#include <signal.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
-#include <pwd.h>
 #include <sys/param.h>
-#include <fcntl.h>
-#include <stdbool.h>
 #include <time.h>
-#include <poll.h>
-#include <dirent.h>
-#include <signal.h>
 #include <unistd.h>
-#include <glib.h>
-#include <gio/gio.h>
-
-#include "soundgapless.h"
-#include "stringfunc.h"
-#include "settings.h"
-#include "cutils.h"
-#include "playlist.h"
+#include "cache.h"
 #include "events.h"
 #include "file.h"
-#include "visuals.h"
-#include "albumart.h"
-#include "player.h"
-#include "cache.h"
-#include "songloader.h"
-#include "volume.h"
-#include "playerops.h"
 #include "mpris.h"
+#include "player.h"
+#include "playerops.h"
+#include "playlist.h"
+#include "settings.h"
+#include "sound.h"
 #include "soundcommon.h"
+#include "songloader.h"
+#include "utils.h"
 
 // #define DEBUG 1
 #define MAX_SEQ_LEN 1024    // Maximum length of sequence buffer
 #define MAX_TMP_SEQ_LEN 256 // Maximum length of temporary sequence buffer
 #define COOLDOWN_MS 500
 #define COOLDOWN2_MS 100
-#define DELAYEDACTIONWAIT 3000
-#define NUMKEYMAPPINGS = 30
+
 FILE *logFile = NULL;
 struct winsize windowSize;
 static bool eventProcessed = false;
 char digitsPressed[MAX_SEQ_LEN];
 int digitsPressedCount = 0;
 int maxDigitsPressedCount = 9;
-bool gotoSong = false;
 bool gPressed = false;
 bool loadingAudioData = false;
 bool goingToSong = false;
 bool startFromTop = false;
+bool exactSearch = false;
+AppSettings settings;
 
 bool isCooldownElapsed(int milliSeconds)
 {
@@ -93,19 +89,6 @@ bool isCooldownElapsed(int milliSeconds)
         return elapsedMilliseconds >= milliSeconds;
 }
 
-void performDelayedActions()
-{
-        struct timespec currentTime;
-        clock_gettime(CLOCK_MONOTONIC, &currentTime);
-        double elapsedMilliseconds = (currentTime.tv_sec - lastPlaylistChangeTime.tv_sec) * 1000.0 +
-                                     (currentTime.tv_nsec - lastPlaylistChangeTime.tv_nsec) / 1000000.0;
-
-        if (elapsedMilliseconds >= DELAYEDACTIONWAIT)
-        {
-                rebuildAndUpdatePlaylist();
-        }
-}
-
 struct Event processInput()
 {
         struct Event event;
@@ -114,9 +97,6 @@ struct Event processInput()
         bool cooldownElapsed = false;
         bool cooldown2Elapsed = false;
 
-        if (songLoading)
-                return event;
-
         if (!isInputAvailable())
         {
                 flushSeek();
@@ -185,7 +165,7 @@ struct Event processInput()
                                       {settings.nextTrackAlt, EVENT_NEXT},
                                       {settings.previousTrackAlt, EVENT_PREV},
                                       {settings.volumeUp, EVENT_VOLUME_UP},
-                                      {settings.volumeUpAlt, EVENT_VOLUME_UP},                                      
+                                      {settings.volumeUpAlt, EVENT_VOLUME_UP},
                                       {settings.volumeDown, EVENT_VOLUME_DOWN},
                                       {settings.togglePause, EVENT_PLAY_PAUSE},
                                       {settings.quit, EVENT_QUIT},
@@ -200,6 +180,7 @@ struct Event processInput()
                                       {settings.savePlaylist, EVENT_EXPORTPLAYLIST},
                                       {settings.toggleColorsDerivedFrom, EVENT_TOGGLE_PROFILE_COLORS},
                                       {settings.addToMainPlaylist, EVENT_ADDTOMAINPLAYLIST},
+                                      {settings.updateLibrary, EVENT_UPDATELIBRARY},
                                       {settings.hardPlayPause, EVENT_PLAY_PAUSE},
                                       {settings.hardPrev, EVENT_PREV},
                                       {settings.hardNext, EVENT_NEXT},
@@ -316,81 +297,21 @@ struct Event processInput()
         return event;
 }
 
-void unloadPreviousSong()
-{
-        pthread_mutex_lock(&(loadingdata.mutex));
-
-        if (usingSongDataA &&
-            (skipping || (userData.currentSongData == NULL || userData.currentSongData->deleted == true ||
-                          (loadingdata.songdataA != NULL && loadingdata.songdataA->deleted == false && userData.currentSongData->trackId != NULL && strcmp(loadingdata.songdataA->trackId, userData.currentSongData->trackId) != 0))))
-        {
-                unloadSongData(&loadingdata.songdataA);
-                userData.filenameA = NULL;
-                usingSongDataA = false;
-                if (!audioData.endOfListReached)
-                        loadedNextSong = false;
-        }
-        else if (!usingSongDataA &&
-                 (skipping || (userData.currentSongData == NULL || userData.currentSongData->deleted == true ||
-                               (loadingdata.songdataB != NULL && loadingdata.songdataB->deleted == false && userData.currentSongData->trackId != NULL && strcmp(loadingdata.songdataB->trackId, userData.currentSongData->trackId) != 0))))
-        {
-                unloadSongData(&loadingdata.songdataB);
-                userData.filenameB = NULL;
-                usingSongDataA = true;
-                if (!audioData.endOfListReached)
-                        loadedNextSong = false;
-        }
-
-        pthread_mutex_unlock(&(loadingdata.mutex));
-}
-
 void setEndOfListReached()
 {
         appState.currentView = LIBRARY_VIEW;
-
         loadedNextSong = false;
-
         audioData.endOfListReached = true;
-
         usingSongDataA = false;
-
         audioData.currentFileIndex = 0;
-
         audioData.restart = true;
-
         loadingdata.loadA = true;
-
-        SongData *songData = (audioData.currentFileIndex == 0) ? userData.songdataA : userData.songdataB;
-
-        if (songData != NULL && songData->deleted == false)
-        {
-                pthread_mutex_lock(&(loadingdata.mutex));
-                unloadSongData(&songData);
-                pthread_mutex_unlock(&(loadingdata.mutex));
-        }
-
-        emitMetadataChanged("", "", "", "", "/org/mpris/MediaPlayer2/TrackList/NoTrack", NULL);
-
-        emitPlaybackStoppedMpris();                
-
+        emitMetadataChanged("", "", "", "", "/org/mpris/MediaPlayer2/TrackList/NoTrack", NULL, 0);
+        emitPlaybackStoppedMpris();
         pthread_mutex_lock(&dataSourceMutex);
-
         cleanupPlaybackDevice();
-
-        if (audioData.fileA != NULL)
-                fclose(audioData.fileA);
-
-        audioData.fileA = NULL;
-
-        if (audioData.fileB != NULL)
-                fclose(audioData.fileB);
-
-        audioData.fileB = NULL;        
-
         pthread_mutex_unlock(&dataSourceMutex);
-
         refresh = true;
-
         chosenRow = playlist.count - 1;
 }
 
@@ -415,9 +336,18 @@ void resetTimeCount()
         totalPauseSeconds = 0.0;
 }
 
+void notifySongSwitch()
+{
+        SongData *currentSongData = (audioData.currentFileIndex == 0) ? userData.songdataA : userData.songdataB;
+        bool isDeleted = (audioData.currentFileIndex == 0) ? userData.songdataADeleted == true : userData.songdataBDeleted == true;
+
+        if (isDeleted == false && currentSongData != NULL && currentSongData->hasErrors == 0 && currentSongData->metadata && strlen(currentSongData->metadata->title) > 0)
+                displaySongNotification(currentSongData->metadata->artist, currentSongData->metadata->title, currentSongData->coverArtPath);
+}
+
 void prepareNextSong()
 {
-        if (!skipPrev && !gotoSong && !isRepeatEnabled())
+        if (!skipOutOfOrder && !isRepeatEnabled())
         {
                 if (currentSong != NULL)
                         lastPlayedId = currentSong->id;
@@ -425,8 +355,7 @@ void prepareNextSong()
         }
         else
         {
-                skipPrev = false;
-                gotoSong = false;
+                skipOutOfOrder = false;
         }
 
         finishLoading();
@@ -435,7 +364,7 @@ void prepareNextSong()
         nextSong = NULL;
         refresh = true;
 
-        if (!isRepeatEnabled())
+        if (!isRepeatEnabled() || currentSong == NULL)
         {
                 unloadPreviousSong();
         }
@@ -447,30 +376,47 @@ void prepareNextSong()
                 else
                         setEndOfListReached();
         }
+        else
+        {
+                notifySongSwitch();
+        }
 
         clock_gettime(CLOCK_MONOTONIC, &start_time);
 }
 
 void refreshPlayer()
 {
+        if (pthread_mutex_trylock(&switchMutex) != 0)
+        {
+                return;
+        }
+
         SongData *songData = (audioData.currentFileIndex == 0) ? userData.songdataA : userData.songdataB;
+        bool isDeleted = (audioData.currentFileIndex == 0) ? userData.songdataADeleted == true : userData.songdataBDeleted == true;
 
         if (!skipping && !isEOFReached() && !isImplSwitchReached())
         {
-                if (refresh && songData != NULL && songData->deleted == false &&
+                if (refresh && songData != NULL && isDeleted == false &&
                     songData->hasErrors == false && currentSong != NULL && songData->metadata != NULL)
                 {
+                        gint64 length = llround(songData->duration * G_USEC_PER_SEC);
                         // update mpris
                         emitMetadataChanged(
                             songData->metadata->title,
                             songData->metadata->artist,
                             songData->metadata->album,
                             songData->coverArtPath,
-                            songData->trackId != NULL ? songData->trackId : "", currentSong);
+                            songData->trackId != NULL ? songData->trackId : "", currentSong,
+                            length);
                 }
 
-                printPlayer(songData, elapsedSeconds, &playlist);
+                if (isDeleted || songData->hasErrors)
+                        songData = NULL;
+
+                printPlayer(songData, elapsedSeconds, &settings);
         }
+
+        pthread_mutex_unlock(&switchMutex);
 }
 
 void handleGoToSong()
@@ -511,19 +457,24 @@ void handleGoToSong()
                 {
                         Node *lastSong = findSelectedEntryById(&playlist, lastPlayedId);
                         startFromTop = false;
-                        
+
                         if (lastSong == NULL)
                         {
-                             if (playlist.tail != NULL)
-                                lastPlayedId = playlist.tail->id;
-                             else
-                             {
-                                lastPlayedId = -1;
-                                startFromTop = true;
-                             }
+                                if (playlist.tail != NULL)
+                                        lastPlayedId = playlist.tail->id;
+                                else
+                                {
+                                        lastPlayedId = -1;
+                                        startFromTop = true;
+                                }
                         }
                 }
+
+                pthread_mutex_lock(&(playlist.mutex));
+
                 enqueueSongs();
+
+                pthread_mutex_unlock(&(playlist.mutex));
         }
 
         goingToSong = false;
@@ -568,22 +519,22 @@ void handleInput()
                 togglePause(&totalPauseSeconds, &pauseSeconds, &pause_time);
                 break;
         case EVENT_TOGGLEVISUALIZER:
-                toggleVisualizer();
+                toggleVisualizer(&settings);
                 break;
         case EVENT_TOGGLEREPEAT:
                 toggleRepeat();
                 break;
         case EVENT_TOGGLECOVERS:
-                toggleCovers();
+                toggleCovers(&settings);
                 break;
         case EVENT_TOGGLEBLOCKS:
-                toggleBlocks();
+                toggleBlocks(&settings);
                 break;
         case EVENT_SHUFFLE:
                 toggleShuffle();
                 break;
         case EVENT_TOGGLE_PROFILE_COLORS:
-                toggleColors();
+                toggleColors(&settings);
                 break;
         case EVENT_QUIT:
                 quit();
@@ -615,13 +566,13 @@ void handleInput()
                 seekForward();
                 break;
         case EVENT_ADDTOMAINPLAYLIST:
-                addToPlaylist();
-                break;
-        case EVENT_DELETEFROMMAINPLAYLIST:
-                // FIXME implement this
+                addToSpecialPlaylist();
                 break;
         case EVENT_EXPORTPLAYLIST:
-                savePlaylist();
+                savePlaylist(settings.path);
+                break;
+        case EVENT_UPDATELIBRARY:
+                updateLibrary(settings.path);
                 break;
         case EVENT_SHOWKEYBINDINGS:
                 toggleShowKeyBindings();
@@ -640,6 +591,7 @@ void handleInput()
                 break;
         case EVENT_REMOVE:
                 handleRemove();
+                break;
         case EVENT_SHOWTRACK:
                 showTrack();
                 break;
@@ -724,31 +676,41 @@ void loadAudioData()
                         waitingForPlaylist = false;
                         waitingForNext = false;
 
-                        loadFirst(currentSong);
+                        int res = loadFirst(currentSong);
 
-                        createAudioDevice(&userData);
+                        if (res >= 0)
+                        {
+                                res = createAudioDevice(&userData);
+                        }
 
-                        resumePlayback();
+                        if (res >= 0)
+                        {
+                                resumePlayback();                            
+                        }
+                        else
+                        {
+                                setEndOfListReached();
+                        }
 
+                        playlistNeedsUpdate = false;
                         loadedNextSong = false;
                         nextSong = NULL;
                         refresh = true;
 
-                        clock_gettime(CLOCK_MONOTONIC, &start_time);
-
-                        playlist.totalDuration = -1;
-                        playlistNeedsUpdate = false;
+                        clock_gettime(CLOCK_MONOTONIC, &start_time);                        
                 }
         }
-        else if (nextSong == NULL && !songLoading)
+        else if (currentSong != NULL && (nextSongNeedsRebuilding || nextSong == NULL) && !songLoading)
         {
-                tryNextSong = currentSong->next;
                 songLoading = true;
+                nextSongNeedsRebuilding = false;
+                tryNextSong = currentSong->next;
                 loadingdata.loadA = !usingSongDataA;
                 nextSong = getListNext(currentSong);
+                loadingdata.loadingFirstDecoder = false;
                 loadSong(nextSong, &loadingdata);
         }
-        
+
         loadingAudioData = false;
 }
 
@@ -766,6 +728,7 @@ void tryLoadNext()
         {
                 songLoading = true;
                 loadingdata.loadA = !usingSongDataA;
+                loadingdata.loadingFirstDecoder = false;
                 loadSong(tryNextSong, &loadingdata);
         }
         else
@@ -776,6 +739,8 @@ void tryLoadNext()
 
 gboolean mainloop_callback(gpointer data)
 {
+        (void)data;
+        
         calcElapsedTime();
         handleInput();
 
@@ -786,12 +751,13 @@ gboolean mainloop_callback(gpointer data)
         }
 
         updatePlayer();
-        performDelayedActions();
 
         if (playlist.head != NULL)
         {
-                if (loadingAudioData == false && !loadedNextSong && !audioData.endOfListReached)
+                if (loadingAudioData == false && (!loadedNextSong || nextSongNeedsRebuilding) && !audioData.endOfListReached)
+                {
                         loadAudioData();
+                }
 
                 if (songHasErrors)
                         tryLoadNext();
@@ -814,20 +780,39 @@ gboolean mainloop_callback(gpointer data)
         return TRUE;
 }
 
+static gboolean on_sigint(gpointer user_data)
+{
+        doQuit = true;
+        GMainLoop *loop = (GMainLoop *)user_data;
+        g_main_loop_quit(loop);
+        return G_SOURCE_REMOVE; // Remove the signal source
+}
+
 void play(Node *song)
 {
         updateLastInputTime();
         updateLastSongSwitchTime();
 
+        int res = 0;
+
         if (song != NULL)
         {
                 audioData.currentFileIndex = 0;
                 loadingdata.loadA = true;
-                loadFirst(song);
-                createAudioDevice(&userData);
+                res = loadFirst(song);
+
+                if (res >= 0)
+                {
+                        res = createAudioDevice(&userData);
+                }
+
+                if (res < 0)
+                        setEndOfListReached();
         }
-        else
+                
+        if (song == NULL || res < 0)
         {
+                song = NULL;
                 waitingForPlaylist = true;
         }
 
@@ -836,11 +821,11 @@ void play(Node *song)
         refresh = true;
 
         clock_gettime(CLOCK_MONOTONIC, &start_time);
-        
-        playlist.totalDuration = -1;
 
         main_loop = g_main_loop_new(NULL, FALSE);
 
+        g_unix_signal_add(SIGINT, on_sigint, main_loop);
+
         if (song != NULL)
                 emitStartPlayingMpris();
         else
@@ -857,29 +842,49 @@ void cleanupOnExit()
         resetDecoders();
         resetVorbisDecoders();
         resetOpusDecoders();
+        resetM4aDecoders();
         cleanupPlaybackDevice();
         cleanupAudioContext();
         emitPlaybackStoppedMpris();
-        unloadSongData(&loadingdata.songdataA);
-        unloadSongData(&loadingdata.songdataB);
-        stopFFmpeg();
+        resetConsole();
+
+        if (library == NULL || library->children == NULL)
+        {
+                printf("No Music found.\n");
+                printf("Please make sure the path is set correctly. \n");
+                printf("To set it type: kew path \"/path/to/Music\". \n");                
+        }
+
+        if (!userData.songdataADeleted)
+        {
+                userData.songdataADeleted = true;
+                unloadSongData(&loadingdata.songdataA);
+        }
+        if (!userData.songdataBDeleted)
+        {
+                userData.songdataBDeleted = true;
+                unloadSongData(&loadingdata.songdataB);
+        }
+
         cleanupMpris();
         restoreTerminalMode();
         enableInputBuffering();
-        setConfig();
-        saveMainPlaylist(settings.path, playingMainPlaylist);
+        setConfig(&settings);
+        saveSpecialPlaylist(settings.path, playingMainPlaylist);
         freeAudioBuffer();
         deleteCache(tempCache);
         deleteTempDir();
         freeMainDirectoryTree();
         deletePlaylist(&playlist);
-        deletePlaylist(mainPlaylist);
-        free(mainPlaylist);
+        deletePlaylist(originalPlaylist);
+        deletePlaylist(specialPlaylist);
+        free(specialPlaylist);
         free(originalPlaylist);
-        cleanupAudioData();
+        setDefaultTextColor();
         showCursor();
         pthread_mutex_destroy(&(loadingdata.mutex));
         pthread_mutex_destroy(&(playlist.mutex));
+        pthread_mutex_destroy(&(switchMutex));
         pthread_mutex_unlock(&dataSourceMutex);
         pthread_mutex_destroy(&(dataSourceMutex));
 
@@ -914,8 +919,7 @@ void run()
 }
 
 void init()
-{
-        clearScreen();
+{       
         disableInputBuffering();
         srand(time(NULL));
         initResize();
@@ -927,14 +931,19 @@ void init()
         loadingdata.songdataA = NULL;
         loadingdata.songdataB = NULL;
         loadingdata.loadA = true;
+        loadingdata.loadingFirstDecoder = true;
         audioData.restart = true;
+        userData.songdataADeleted = true;
+        userData.songdataBDeleted = true;
         initAudioBuffer();
         initVisuals();
         pthread_mutex_init(&dataSourceMutex, NULL);
+        pthread_mutex_init(&switchMutex, NULL);
         pthread_mutex_init(&(loadingdata.mutex), NULL);
         pthread_mutex_init(&(playlist.mutex), NULL);
         nerdFontsEnabled = hasNerdFonts();
-        createLibrary();
+        fflush(stdout);
+        createLibrary(&settings);
 
 #ifdef DEBUG
         g_setenv("G_MESSAGES_DEBUG", "all", TRUE);
@@ -948,6 +957,7 @@ void init()
         (void)nullStream;
 #endif
 }
+
 void openLibrary()
 {
         appState.currentView = LIBRARY_VIEW;
@@ -956,30 +966,31 @@ void openLibrary()
         run();
 }
 
-void playMainPlaylist()
+void playSpecialPlaylist()
 {
-        if (mainPlaylist->count == 0)
+        if (specialPlaylist->count == 0)
         {
-                printf("Couldn't find any songs in the main playlist. Add a song by pressing '.' while it's playing. \n");
+                printf("Couldn't find any songs in the special playlist. Add a song by pressing '.' while it's playing. \n");
                 exit(0);
         }
-        init();
+        
         playingMainPlaylist = true;
-        playlist = deepCopyPlayList(mainPlaylist);
+
+        init();
+        deepCopyPlayListOntoList(specialPlaylist, &playlist);
         shufflePlaylist(&playlist);
         run();
 }
 
-void playAll(int argc, char **argv)
+void playAll()
 {
         init();
-        makePlaylist(argc, argv);
+        createPlayListFromFileSystemEntry(library, &playlist, MAX_FILES);
         if (playlist.count == 0)
         {
-                printf("Please make sure the path is set correctly. \n");
-                printf("To set it type: kew path \"/path/to/Music\". \n");
                 exit(0);
         }
+        shufflePlaylist(&playlist);
         run();
 }
 
@@ -1006,6 +1017,9 @@ void handleOptions(int *argc, char *argv[])
         const char *noUiOption = "--noui";
         const char *noCoverOption = "--nocover";
         const char *quitOnStop = "--quitonstop";
+        const char *quitOnStop2 = "-q";
+        const char *exactOption = "--exact";
+        const char *exactOption2 = "-e";
 
         int idx = -1;
         for (int i = 0; i < *argc; i++)
@@ -1034,7 +1048,7 @@ void handleOptions(int *argc, char *argv[])
         idx = -1;
         for (int i = 0; i < *argc; i++)
         {
-                if (c_strcasestr(argv[i], quitOnStop))
+                if (c_strcasestr(argv[i], quitOnStop) || c_strcasestr(argv[i], quitOnStop2))
                 {
                         quitAfterStopping = true;
                         idx = i;
@@ -1042,29 +1056,53 @@ void handleOptions(int *argc, char *argv[])
         }
         if (idx >= 0)
                 removeArgElement(argv, idx, argc);
+
+        idx = -1;
+        for (int i = 0; i < *argc; i++)
+        {
+                if (c_strcasestr(argv[i], exactOption) || c_strcasestr(argv[i], exactOption2))
+                {
+                        exactSearch = true;
+                        idx = i;
+                }
+        }
+        if (idx >= 0)
+                removeArgElement(argv, idx, argc);
 }
 
 int main(int argc, char *argv[])
 {
-        atexit(cleanupOnExit);
-        getConfig();
+        if ((argc == 2 && ((strcmp(argv[1], "--help") == 0) || (strcmp(argv[1], "-h") == 0) || (strcmp(argv[1], "-?") == 0))))
+        {
+                showHelp();
+                exit(0);
+        }
+        else if (argc == 2 && (strcmp(argv[1], "--version") == 0 || strcmp(argv[1], "-v") == 0))
+        {
+                printAbout(NULL);
+                exit(0);
+        }
+
+        getConfig(&settings);
 
         if (argc == 3 && (strcmp(argv[1], "path") == 0))
         {
                 c_strcpy(settings.path, sizeof(settings.path), argv[2]);
-                setConfig();
+                setConfig(&settings);
                 exit(0);
-        }        
-        else if (settings.path[0] == '\0') 
+        }
+
+        if (settings.path[0] == '\0')
         {
                 printf("Please make sure the path is set correctly. \n");
                 printf("To set it type: kew path \"/path/to/Music\". \n");
                 exit(0);
         }
 
-        loadMainPlaylist(settings.path);
+        atexit(cleanupOnExit);
 
         handleOptions(&argc, argv);
+        loadSpecialPlaylist(settings.path);
 
         if (argc == 1)
         {
@@ -1072,24 +1110,16 @@ int main(int argc, char *argv[])
         }
         else if (argc == 2 && strcmp(argv[1], "all") == 0)
         {
-                playAll(argc, argv);
+                playAll();
         }
         else if (argc == 2 && strcmp(argv[1], ".") == 0)
         {
-                playMainPlaylist();
-        }
-        else if ((argc == 2 && ((strcmp(argv[1], "--help") == 0) || (strcmp(argv[1], "-h") == 0) || (strcmp(argv[1], "-?") == 0))))
-        {
-                showHelp();
-        }
-        else if (argc == 2 && (strcmp(argv[1], "--version") == 0 || strcmp(argv[1], "-v") == 0))
-        {
-                printAbout(NULL);
+                playSpecialPlaylist();
         }
         else if (argc >= 2)
         {
                 init();
-                makePlaylist(argc, argv);
+                makePlaylist(argc, argv, exactSearch, settings.path);
                 if (playlist.count == 0)
                         exit(0);
                 run();
