@@ -16,7 +16,6 @@ guint bus_name_id;
 static const gchar *LoopStatus = "None";
 static gdouble Rate = 1.0;
 static gdouble Volume = 0.5;
-static gint64 Position = 0;
 static gdouble MinimumRate = 1.0;
 static gdouble MaximumRate = 1.0;
 static gboolean CanGoNext = TRUE;
@@ -97,6 +96,14 @@ static void handle_raise(GDBusConnection *connection, const gchar *sender,
                          const gchar *method_name, GVariant *parameters,
                          GDBusMethodInvocation *invocation, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)invocation;
+        (void)user_data;
 }
 
 static void handle_quit(GDBusConnection *connection, const gchar *sender,
@@ -104,6 +111,15 @@ static void handle_quit(GDBusConnection *connection, const gchar *sender,
                         const gchar *method_name, GVariant *parameters,
                         GDBusMethodInvocation *invocation, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)invocation;
+        (void)user_data;
+
         quit();
 }
 
@@ -112,6 +128,14 @@ static gboolean get_identity(GDBusConnection *connection, const gchar *sender,
                              const gchar *property_name, GVariant **value,
                              GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_string(identity);
         return TRUE;
 }
@@ -121,6 +145,14 @@ static gboolean get_desktop_entry(GDBusConnection *connection, const gchar *send
                                   const gchar *property_name, GVariant **value,
                                   GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_string(desktopEntry);
         return TRUE;
 }
@@ -130,6 +162,14 @@ static gboolean get_desktop_icon_name(GDBusConnection *connection, const gchar *
                                       const gchar *property_name, GVariant **value,
                                       GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_string(desktopIconName);
         return TRUE;
 }
@@ -139,6 +179,14 @@ static void handle_next(GDBusConnection *connection, const gchar *sender,
                         const gchar *method_name, GVariant *parameters,
                         GDBusMethodInvocation *invocation, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)user_data;
+
         skipToNextSong();
         g_dbus_method_invocation_return_value(invocation, NULL);
 }
@@ -148,6 +196,14 @@ static void handle_previous(GDBusConnection *connection, const gchar *sender,
                             const gchar *method_name, GVariant *parameters,
                             GDBusMethodInvocation *invocation, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)user_data;
+
         skipToPrevSong();
         g_dbus_method_invocation_return_value(invocation, NULL);
 }
@@ -157,7 +213,16 @@ static void handle_pause(GDBusConnection *connection, const gchar *sender,
                          const gchar *method_name, GVariant *parameters,
                          GDBusMethodInvocation *invocation, gpointer user_data)
 {
-        playbackPause(&totalPauseSeconds, &pauseSeconds, &pause_time);
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)invocation;
+        (void)user_data;
+
+        playbackPause(&pause_time);
 }
 
 static void handle_play_pause(GDBusConnection *connection, const gchar *sender,
@@ -165,6 +230,14 @@ static void handle_play_pause(GDBusConnection *connection, const gchar *sender,
                               const gchar *method_name, GVariant *parameters,
                               GDBusMethodInvocation *invocation, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)user_data;
+
         togglePause(&totalPauseSeconds, &pauseSeconds, &pause_time);
         g_dbus_method_invocation_return_value(invocation, NULL);
 }
@@ -174,6 +247,14 @@ static void handle_stop(GDBusConnection *connection, const gchar *sender,
                         const gchar *method_name, GVariant *parameters,
                         GDBusMethodInvocation *invocation, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)user_data;
+
         quit();
         g_dbus_method_invocation_return_value(invocation, NULL);
 }
@@ -183,7 +264,16 @@ static void handle_play(GDBusConnection *connection, const gchar *sender,
                         const gchar *method_name, GVariant *parameters,
                         GDBusMethodInvocation *invocation, gpointer user_data)
 {
-        playbackPlay(&totalPauseSeconds, &pauseSeconds, &pause_time);
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)invocation;
+        (void)user_data;
+
+        playbackPlay(&totalPauseSeconds, &pauseSeconds);
 }
 
 static void handle_seek(GDBusConnection *connection, const gchar *sender,
@@ -191,6 +281,14 @@ static void handle_seek(GDBusConnection *connection, const gchar *sender,
                         const gchar *method_name, GVariant *parameters,
                         GDBusMethodInvocation *invocation, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)invocation;        
+        (void)user_data;        
 }
 
 static void handle_set_position(GDBusConnection *connection, const gchar *sender,
@@ -198,6 +296,14 @@ static void handle_set_position(GDBusConnection *connection, const gchar *sender
                                 const gchar *method_name, GVariant *parameters,
                                 GDBusMethodInvocation *invocation, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)method_name;
+        (void)parameters;
+        (void)invocation;
+        (void)user_data;   
 }
 
 static void handle_method_call(GDBusConnection *connection, const gchar *sender,
@@ -263,14 +369,20 @@ static void handle_method_call(GDBusConnection *connection, const gchar *sender,
         }
 }
 
+
 static void on_bus_name_acquired(GDBusConnection *connection, const gchar *name, gpointer user_data)
 {
-        // g_print("Acquired bus name: %s\n", name);
+        (void)connection;
+        (void)name;
+        (void)user_data;
+
 }
 
 static void on_bus_name_lost(GDBusConnection *connection, const gchar *name, gpointer user_data)
 {
-        // g_print("Lost bus name: %s\n", name);
+        (void)connection;
+        (void)name;
+        (void)user_data;
 }
 
 static gboolean get_playback_status(GDBusConnection *connection, const gchar *sender,
@@ -278,6 +390,14 @@ static gboolean get_playback_status(GDBusConnection *connection, const gchar *se
                                     const gchar *property_name, GVariant **value,
                                     GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         const gchar *status = "Stopped";
 
         if (isPaused())
@@ -301,6 +421,14 @@ static gboolean get_loop_status(GDBusConnection *connection, const gchar *sender
                                 const gchar *property_name, GVariant **value,
                                 GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_string(LoopStatus);
         return TRUE;
 }
@@ -310,6 +438,14 @@ static gboolean get_rate(GDBusConnection *connection, const gchar *sender,
                          const gchar *property_name, GVariant **value,
                          GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_double(Rate);
         return TRUE;
 }
@@ -319,6 +455,14 @@ static gboolean get_shuffle(GDBusConnection *connection, const gchar *sender,
                             const gchar *property_name, GVariant **value,
                             GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_boolean(isShuffleEnabled() ? TRUE : FALSE);
         return TRUE;
 }
@@ -328,6 +472,14 @@ static gboolean get_metadata(GDBusConnection *connection, const gchar *sender,
                              const gchar *property_name, GVariant **value,
                              GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         SongData *currentSongData = getCurrentSongData();
 
         GVariantBuilder metadata_builder;
@@ -349,11 +501,19 @@ static gboolean get_metadata(GDBusConnection *connection, const gchar *sender,
                         artistList[0] = "";
                         artistList[1] = NULL;
                 }
+
+                gchar *coverArtUrl = g_strdup_printf("file://%s", currentSongData->coverArtPath);
+
                 g_variant_builder_add(&metadata_builder, "{sv}", "xesam:artist", g_variant_new_strv(artistList, -1));
                 g_variant_builder_add(&metadata_builder, "{sv}", "xesam:album", g_variant_new_string(currentSongData->metadata->album));
                 g_variant_builder_add(&metadata_builder, "{sv}", "xesam:contentCreated", g_variant_new_string(currentSongData->metadata->date));
-                g_variant_builder_add(&metadata_builder, "{sv}", "mpris:artUrl", g_variant_new_string(currentSongData->coverArtPath));
+                g_variant_builder_add(&metadata_builder, "{sv}", "mpris:artUrl", g_variant_new_string(coverArtUrl));
                 g_variant_builder_add(&metadata_builder, "{sv}", "mpris:trackid", g_variant_new_object_path(currentSongData->trackId));
+
+                gint64 length = llround(currentSongData->duration * G_USEC_PER_SEC);
+                g_variant_builder_add(&metadata_builder, "{sv}", "mpris:length", g_variant_new_int64(length));
+
+                g_free(coverArtUrl);
         }
         else
         {
@@ -363,6 +523,9 @@ static gboolean get_metadata(GDBusConnection *connection, const gchar *sender,
                 g_variant_builder_add(&metadata_builder, "{sv}", "xesam:contentCreated", g_variant_new_string(""));
                 g_variant_builder_add(&metadata_builder, "{sv}", "mpris:artUrl", g_variant_new_string(""));
                 g_variant_builder_add(&metadata_builder, "{sv}", "mpris:trackid", g_variant_new_object_path("/org/mpris/MediaPlayer2/TrackList/NoTrack"));
+
+                gint64 placeholderLength = 0;
+                g_variant_builder_add(&metadata_builder, "{sv}", "mpris:length", g_variant_new_int64(placeholderLength));
         }
 
         GVariant *metadata_variant = g_variant_builder_end(&metadata_builder);
@@ -375,6 +538,13 @@ static gboolean get_volume(GDBusConnection *connection, const gchar *sender,
                            const gchar *property_name, GVariant **value,
                            GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;        
 
         Volume = (gdouble)getCurrentVolume();
 
@@ -387,7 +557,19 @@ static gboolean get_position(GDBusConnection *connection, const gchar *sender,
                              const gchar *property_name, GVariant **value,
                              GError **error, gpointer user_data)
 {
-        *value = g_variant_new_int64(Position);
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
+        // Convert elapsedSeconds from milliseconds to microseconds
+        gint64 positionMicroseconds = llround(elapsedSeconds * G_USEC_PER_SEC);
+
+        *value = g_variant_new_int64(positionMicroseconds);
+
         return TRUE;
 }
 
@@ -396,6 +578,14 @@ static gboolean get_minimum_rate(GDBusConnection *connection, const gchar *sende
                                  const gchar *property_name, GVariant **value,
                                  GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_double(MinimumRate);
         return TRUE;
 }
@@ -405,6 +595,14 @@ static gboolean get_maximum_rate(GDBusConnection *connection, const gchar *sende
                                  const gchar *property_name, GVariant **value,
                                  GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_double(MaximumRate);
         return TRUE;
 }
@@ -414,6 +612,13 @@ static gboolean get_can_go_next(GDBusConnection *connection, const gchar *sender
                                 const gchar *property_name, GVariant **value,
                                 GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
 
         CanGoNext = (currentSong == NULL || currentSong->next != NULL) ? TRUE : FALSE;
 
@@ -426,6 +631,13 @@ static gboolean get_can_go_previous(GDBusConnection *connection, const gchar *se
                                     const gchar *property_name, GVariant **value,
                                     GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;        
 
         CanGoPrevious = (currentSong == NULL || currentSong->prev != NULL) ? TRUE : FALSE;
 
@@ -438,6 +650,14 @@ static gboolean get_can_play(GDBusConnection *connection, const gchar *sender,
                              const gchar *property_name, GVariant **value,
                              GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_boolean(CanPlay);
         return TRUE;
 }
@@ -447,6 +667,14 @@ static gboolean get_can_pause(GDBusConnection *connection, const gchar *sender,
                               const gchar *property_name, GVariant **value,
                               GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_boolean(CanPause);
         return TRUE;
 }
@@ -456,6 +684,14 @@ static gboolean get_can_seek(GDBusConnection *connection, const gchar *sender,
                              const gchar *property_name, GVariant **value,
                              GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_boolean(CanSeek);
         return TRUE;
 }
@@ -465,6 +701,14 @@ static gboolean get_can_control(GDBusConnection *connection, const gchar *sender
                                 const gchar *property_name, GVariant **value,
                                 GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)error;
+        (void)user_data;
+
         *value = g_variant_new_boolean(CanControl);
         return TRUE;
 }
@@ -567,6 +811,12 @@ static gboolean set_property_callback(GDBusConnection *connection, const gchar *
                                       const gchar *property_name, GVariant *value,
                                       GError **error, gpointer user_data)
 {
+        (void)connection;
+        (void)sender;
+        (void)object_path;
+        (void)interface_name;
+        (void)property_name;
+        (void)user_data;        
 
         if (g_strcmp0(interface_name, "org.mpris.MediaPlayer2.Player") == 0)
         {
@@ -702,11 +952,13 @@ void initMpris()
 
         if (!connection)
         {
+                g_dbus_node_info_unref(introspection_data);
                 g_printerr("Failed to connect to D-Bus\n");
                 exit(0);
         }
 
         const char *app_name = "org.mpris.MediaPlayer2.kew";
+       
         char unique_name[256];
         snprintf(unique_name, sizeof(unique_name), "%s%d", app_name, getpid());
 
@@ -736,6 +988,7 @@ void initMpris()
 
         if (!registration_id)
         {
+                g_dbus_node_info_unref(introspection_data);
                 g_printerr("Failed to register media player object: %s\n", error->message);
                 g_error_free(error);
                 exit(0);
@@ -752,10 +1005,13 @@ void initMpris()
 
         if (!player_registration_id)
         {
+                g_dbus_node_info_unref(introspection_data);
                 g_printerr("Failed to register media player object: %s\n", error->message);
                 g_error_free(error);
                 exit(0);
         }
+
+        g_dbus_node_info_unref(introspection_data);
 }
 
 void emitStartPlayingMpris()
@@ -770,17 +1026,14 @@ void emitStartPlayingMpris()
                                       NULL);
 }
 
-void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *album, const gchar *coverArtPath, const gchar *trackId, Node *currentSong)
+void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *album, const gchar *coverArtPath, const gchar *trackId, Node *currentSong, gint64 length)
 {
-        // Convert the coverArtPath to a valid URL format
         gchar *coverArtUrl = g_strdup_printf("file://%s", coverArtPath);
 
-        // Create a GVariantBuilder for the metadata dictionary
         GVariantBuilder metadata_builder;
         g_variant_builder_init(&metadata_builder, G_VARIANT_TYPE_DICTIONARY);
         g_variant_builder_add(&metadata_builder, "{sv}", "xesam:title", g_variant_new_string(title));
 
-        // Build list of strings for artist
         const gchar *artistList[2];
         if (artist)
         {
@@ -797,6 +1050,7 @@ void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *a
         g_variant_builder_add(&metadata_builder, "{sv}", "xesam:album", g_variant_new_string(album));
         g_variant_builder_add(&metadata_builder, "{sv}", "mpris:artUrl", g_variant_new_string(coverArtUrl));
         g_variant_builder_add(&metadata_builder, "{sv}", "mpris:trackid", g_variant_new_object_path(trackId));
+        g_variant_builder_add(&metadata_builder, "{sv}", "mpris:length", g_variant_new_int64(length));
 
         GVariantBuilder changed_properties_builder;
         g_variant_builder_init(&changed_properties_builder, G_VARIANT_TYPE("a{sv}"));
@@ -809,4 +1063,6 @@ void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *a
 
         g_variant_builder_clear(&metadata_builder);
         g_variant_builder_clear(&changed_properties_builder);
+
+        g_free(coverArtUrl);
 }
