@@ -1,13 +1,12 @@
 #ifndef MPRIS_H
 #define MPRIS_H
 
-#include <glib.h>
 #include <gio/gio.h>
-#include "songloader.h"
-#include "soundgapless.h"
-#include "volume.h"
-#include "playlist.h"
+#include <glib.h>
 #include "playerops.h"
+#include "playlist.h"
+#include "sound.h"
+#include "soundcommon.h"
 
 extern GDBusConnection *connection;
 extern GMainContext *global_main_context;
@@ -19,7 +18,7 @@ void emitStringPropertyChanged(const gchar *propertyName, const gchar *newValue)
 
 void emitBooleanPropertyChanged(const gchar *propertyName, gboolean newValue);
 
-void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *album, const gchar *coverArtPath, const gchar *trackId, Node *currentSong);
+void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *album, const gchar *coverArtPath, const gchar *trackId, Node *currentSong, gint64 length);
 
 void emitStartPlayingMpris(void);
 
