@@ -1,19 +1,8 @@
 #define _XOPEN_SOURCE 700
 #define __USE_XOPEN_EXTENDED 1
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <strings.h>
-#include <dirent.h>
-#include <regex.h>
-#include <string.h>
-#include <time.h>
-#include <ctype.h>
-#include "cutils.h"
+
 #include "playlist.h"
-#include "file.h"
-#include "stringfunc.h"
-#include "settings.h"
+
 /*
 
 playlist.c
@@ -22,7 +11,6 @@ Playlist related functions.
 
 */
 #define MAX_SEARCH_SIZE 256
-#define MAX_FILES 20000
 
 #ifndef MAXPATHLEN
 #define MAXPATHLEN 4096
@@ -30,10 +18,16 @@ Playlist related functions.
 
 const char PLAYLIST_EXTENSIONS[] = "\\.(m3u)$";
 const char mainPlaylistName[] = "kew.m3u";
-PlayList playlist = {NULL, NULL, 0, 0.0, PTHREAD_MUTEX_INITIALIZER};
-PlayList *mainPlaylist = NULL;
+
+// The playlist unshuffled as it appears in playlist view
 PlayList *originalPlaylist = NULL;
 
+// The (sometimes shuffled) sequence of songs that will be played
+PlayList playlist = {NULL, NULL, 0, PTHREAD_MUTEX_INITIALIZER};
+
+// The playlist from kew.m3u
+PlayList *specialPlaylist = NULL;
+
 char search[MAX_SEARCH_SIZE];
 char playlistName[MAX_SEARCH_SIZE];
 bool shuffle = false;
@@ -52,14 +46,11 @@ Node *getListPrev(Node *node)
         return (node == NULL) ? NULL : node->prev;
 }
 
-void addToList(PlayList *list, SongInfo song, int id)
+void addToList(PlayList *list, Node *newNode)
 {
         if (list->count >= MAX_FILES)
                 return;
-        Node *newNode = (Node *)malloc(sizeof(Node));
-        newNode->song = song;
-        newNode->next = NULL;
-        newNode->id = id;
+
         list->count++;
 
         if (list->head == NULL)
@@ -81,8 +72,15 @@ Node *deleteFromList(PlayList *list, Node *node)
         if (list->head == NULL || node == NULL)
                 return NULL;
 
-        if (node == list->head)
+        if (list->head == node)
+        {
                 list->head = node->next;
+                if (list->head == NULL)
+                {
+                        list->tail = NULL;
+                }
+        }
+
         if (node == list->tail)
                 list->tail = node->prev;
 
@@ -91,6 +89,9 @@ Node *deleteFromList(PlayList *list, Node *node)
         if (node->next != NULL)
                 node->next->prev = node->prev;
 
+        if (node->song.filePath != NULL)
+                free(node->song.filePath);
+
         Node *nextNode = node->next;
 
         free(node);
@@ -107,24 +108,15 @@ void deletePlaylist(PlayList *list)
         while (current != NULL)
         {
                 Node *next = current->next;
-                if (current->song.filePath != NULL)
-                {
-                        free(current->song.filePath);
-                        current->song.filePath = NULL;
-                }
-                if (current->prev != NULL)
-                        current->prev->next = NULL;
+                free(current->song.filePath);
                 free(current);
                 current = next;
-                if (current != NULL && current->prev != NULL)
-                        current->prev = NULL;
         }
 
         // Reset the playlist
         list->head = NULL;
         list->tail = NULL;
         list->count = 0;
-        list->totalDuration = 0.0;
 }
 
 void shufflePlaylist(PlayList *playlist)
@@ -151,7 +143,6 @@ void shufflePlaylist(PlayList *playlist)
         }
 
         // Shuffle the array using Fisher-Yates algorithm
-
         for (int j = playlist->count - 1; j >= 1; --j)
         {
                 int k = rand() % (j + 1);
@@ -191,6 +182,11 @@ void insertAsFirst(Node *currentSong, PlayList *playlist)
                         {
                                 currentSong->next->prev = currentSong->prev;
                         }
+                        else
+                        {
+                                playlist->tail = currentSong->prev;
+                        }
+
                         if (currentSong->prev != NULL)
                         {
                                 currentSong->prev->next = currentSong->next;
@@ -209,7 +205,9 @@ void shufflePlaylistStartingFromSong(PlayList *playlist, Node *song)
 {
         shufflePlaylist(playlist);
         if (song != NULL && playlist->count > 1)
+        {
                 insertAsFirst(song, playlist);
+        }
 }
 
 int compare(const struct dirent **a, const struct dirent **b)
@@ -229,15 +227,34 @@ int compare(const struct dirent **a, const struct dirent **b)
         return strcmp(nameA, nameB);
 }
 
-void buildPlaylistRecursive(char *directoryPath, const char *allowedExtensions, PlayList *playlist)
+void createNode(Node **node, const char *directoryPath, int id)
+{
+        SongInfo song;
+        song.filePath = strdup(directoryPath);
+        song.duration = 0.0;
+
+        *node = (Node *)malloc(sizeof(Node));
+        if (*node == NULL)
+        {
+                printf("Failed to allocate memory.");
+                exit(0);
+                return;
+        }
+
+        (*node)->song = song;
+        (*node)->next = NULL;
+        (*node)->prev = NULL;
+        (*node)->id = id;
+}
+
+void buildPlaylistRecursive(const char *directoryPath, const char *allowedExtensions, PlayList *playlist)
 {
         int res = isDirectory(directoryPath);
         if (res != 1 && res != -1 && directoryPath != NULL)
         {
-                SongInfo song;
-                song.filePath = strdup(directoryPath);
-                song.duration = 0.0;
-                addToList(playlist, song, nodeIdCounter++);
+                Node *node = NULL;
+                createNode(&node, directoryPath, nodeIdCounter++);
+                addToList(playlist, node);
                 return;
         }
 
@@ -291,11 +308,11 @@ void buildPlaylistRecursive(char *directoryPath, const char *allowedExtensions,
                         extractExtension(entry->d_name, sizeof(exto) - 1, exto);
                         if (match_regex(&regex, exto) == 0)
                         {
-                                SongInfo song;
                                 snprintf(filePath, sizeof(filePath), "%s/%s", directoryPath, entry->d_name);
-                                song.filePath = strdup(filePath);
-                                song.duration = 0.0;
-                                addToList(playlist, song, nodeIdCounter++);
+
+                                Node *node = NULL;
+                                createNode(&node, filePath, nodeIdCounter++);
+                                addToList(playlist, node);
                         }
                 }
         }
@@ -334,10 +351,10 @@ int playDirectory(const char *directoryPath, const char *allowedExtensions, Play
                 {
                         char filePath[FILENAME_MAX];
                         snprintf(filePath, sizeof(filePath), "%s/%s", directoryPath, entry->d_name);
-                        SongInfo song;
-                        song.duration = 0.0;
-                        song.filePath = strdup(filePath);
-                        addToList(playlist, song, nodeIdCounter++);
+
+                        Node *node = NULL;
+                        createNode(&node, filePath, nodeIdCounter++);
+                        addToList(playlist, node);
                 }
         }
         closedir(dir);
@@ -389,13 +406,73 @@ void makePlaylistName(const char *search)
         }
 }
 
-int makePlaylist(int argc, char *argv[])
+void readM3UFile(const char *filename, PlayList *playlist)
+{
+        FILE *file = fopen(filename, "r");
+        char directory[MAXPATHLEN];
+
+        if (file == NULL)
+        {
+                return;
+        }
+
+        getDirectoryFromPath(filename, directory);
+        char line[MAXPATHLEN];
+        while (fgets(line, sizeof(line), file))
+        {
+                size_t len = strcspn(line, "\r\n");
+                line[len] = '\0';
+
+                size_t start = 0;
+                while (isspace(line[start]))
+                {
+                        start++;
+                }
+                size_t end = strlen(line);
+                while (end > start && isspace(line[end - 1]))
+                {
+                        end--;
+                }
+                line[end] = '\0';
+
+                if (line[0] != '#' && line[0] != '\0')
+                {
+                        char songPath[MAXPATHLEN];
+                        memset(songPath, '\0', sizeof(songPath));
+
+                        if (strchr(line, '/') == NULL && strchr(line, '\\') == NULL)
+                                strcat(songPath, directory);
+
+                        strcat(songPath, line);
+
+                        Node *newNode = NULL;
+                        createNode(&newNode, songPath, nodeIdCounter++);
+
+                        if (playlist->head == NULL)
+                        {
+                                playlist->head = newNode;
+                                playlist->tail = newNode;
+                        }
+                        else
+                        {
+                                playlist->tail->next = newNode;
+                                newNode->prev = playlist->tail;
+                                playlist->tail = newNode;
+                        }
+
+                        playlist->count++;
+                }
+        }
+        fclose(file);
+}
+
+int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path)
 {
         enum SearchType searchType = SearchAny;
         int searchTypeIndex = 1;
 
         const char *delimiter = ":";
-        PlayList partialPlaylist = {NULL, NULL, 0, 0.0};
+        PlayList partialPlaylist = {NULL, NULL, 0, PTHREAD_MUTEX_INITIALIZER};
 
         const char *allowedExtensions = AUDIO_EXTENSIONS;
 
@@ -455,7 +532,7 @@ int makePlaylist(int argc, char *argv[])
         {
                 pthread_mutex_lock(&(playlist.mutex));
 
-                buildPlaylistRecursive(settings.path, allowedExtensions, &playlist);
+                buildPlaylistRecursive(path, allowedExtensions, &playlist);
 
                 pthread_mutex_unlock(&(playlist.mutex));
         }
@@ -472,7 +549,7 @@ int makePlaylist(int argc, char *argv[])
                                 searchType = FileOnly;
                         }
                         trim(token);
-                        if (walker(settings.path, token, buf, allowedExtensions, searchType) == 0)
+                        if (walker(path, token, buf, allowedExtensions, searchType, exactSearch) == 0)
                         {
                                 if (strcmp(argv[1], "list") == 0)
                                 {
@@ -503,184 +580,6 @@ int makePlaylist(int argc, char *argv[])
         return 0;
 }
 
-void *getDurationsThread(void *arg)
-{
-        PlayList *playList = (PlayList *)arg;
-
-        if (playList == NULL)
-                return NULL;
-
-        pthread_mutex_lock(&(playlist.mutex));
-
-        Node *currentNode = playList->head;
-        double totalDuration = 0.0;
-
-        for (int i = 0; i < playList->count; i++)
-        {
-                if (stopPlaylistDurationThread)
-                {
-                        pthread_mutex_unlock(&(playlist.mutex));
-                        return NULL;
-                }
-                if (currentNode == NULL)
-                {
-                        pthread_mutex_unlock(&(playlist.mutex));
-                        return NULL;
-                }
-                if (currentNode->song.duration > 0.0)
-                {
-                        currentNode = getListNext(currentNode);
-                        continue;
-                }
-                if (currentNode->song.filePath == NULL)
-                {
-                        currentNode = getListNext(currentNode);
-                        continue;
-                }
-                char musicFilepath[MAX_FILENAME_LENGTH];
-                c_strcpy(musicFilepath, sizeof(musicFilepath), currentNode->song.filePath);
-                double duration = ((currentNode->song.duration > 0.0) ? currentNode->song.duration : getDuration(musicFilepath));
-                if (duration > 0.0)
-                {
-                        currentNode->song.duration = duration;
-                        totalDuration += duration;
-                        playList->totalDuration += duration;
-                }
-                currentNode = getListNext(currentNode);
-
-                c_sleep(100);
-        }
-        currentNode = playList->head;
-        playList->totalDuration = 0.0;
-        for (int i = 0; i < playList->count; i++)
-        {
-                if (currentNode != NULL && currentNode->song.duration > 0.0)
-                        playList->totalDuration += currentNode->song.duration;
-                currentNode = getListNext(currentNode);
-        }
-        pthread_mutex_unlock(&(playlist.mutex));
-        return NULL;
-}
-
-double calcTotalDuration(PlayList *playList)
-{
-        double totalDuration = 0.0;
-
-        Node *tmp = playList->head;
-
-        while (tmp != NULL)
-        {
-                totalDuration += tmp->song.duration;
-                tmp = tmp->next;
-        }
-
-        return totalDuration;
-}
-
-int calculatePlayListDuration(PlayList *playlist)
-{
-        if (playlist->count > MAX_COUNT_PLAYLIST_SONGS)
-                return 0;
-
-        Node *tmp = playlist->head;
-        int missingDuration = 0;
-        while (tmp != NULL)
-        {
-                if (tmp->song.duration <= 0.0)
-                {
-                        missingDuration++;
-                }
-                tmp = tmp->next;
-        }
-
-        if (missingDuration == 0)
-                return 0;
-
-        startPlayListDurationCount();
-
-        pthread_t thread;
-        int threadCreationResult;
-        threadCreationResult = pthread_create(&thread, NULL, getDurationsThread, (void *)playlist);
-        if (threadCreationResult != 0)
-        {
-                return 1;
-        }
-        return 0;
-}
-
-void stopPlayListDurationCount()
-{
-        stopPlaylistDurationThread = 1;
-}
-
-void startPlayListDurationCount()
-{
-        stopPlaylistDurationThread = 0;
-}
-
-void readM3UFile(const char *filename, PlayList *playlist)
-{
-        FILE *file = fopen(filename, "r");
-        char directory[MAXPATHLEN];
-
-        if (file == NULL)
-        {
-                return;
-        }
-
-        getDirectoryFromPath(filename, directory);
-        char line[MAXPATHLEN];
-        while (fgets(line, sizeof(line), file))
-        {
-                size_t len = strcspn(line, "\r\n");
-                line[len] = '\0';
-
-                size_t start = 0;
-                while (isspace(line[start]))
-                {
-                        start++;
-                }
-                size_t end = strlen(line);
-                while (end > start && isspace(line[end - 1]))
-                {
-                        end--;
-                }
-                line[end] = '\0';
-
-                if (line[0] != '#' && line[0] != '\0')
-                {
-                        char songPath[MAXPATHLEN];
-                        memset(songPath, '\0', sizeof(songPath));
-
-                        if (strchr(line, '/') == NULL && strchr(line, '\\') == NULL)
-                                strcat(songPath, directory);
-
-                        strcat(songPath, line);
-
-                        Node *newNode = (Node *)malloc(sizeof(Node));
-                        newNode->song.filePath = strdup(songPath);
-                        newNode->song.duration = 0.0;
-                        newNode->next = NULL;
-                        newNode->prev = NULL;
-
-                        if (playlist->head == NULL)
-                        {
-                                playlist->head = newNode;
-                                playlist->tail = newNode;
-                        }
-                        else
-                        {
-                                playlist->tail->next = newNode;
-                                newNode->prev = playlist->tail;
-                                playlist->tail = newNode;
-                        }
-
-                        playlist->count++;
-                }
-        }
-        fclose(file);
-}
-
 void writeM3UFile(const char *filename, PlayList *playlist)
 {
         FILE *file = fopen(filename, "w");
@@ -698,27 +597,26 @@ void writeM3UFile(const char *filename, PlayList *playlist)
         fclose(file);
 }
 
-void loadMainPlaylist(const char *directory)
+void loadSpecialPlaylist(const char *directory)
 {
         char playlistPath[MAXPATHLEN];
         c_strcpy(playlistPath, sizeof(playlistPath), directory);
         if (playlistPath[strlen(playlistPath) - 1] != '/')
                 strcat(playlistPath, "/");
         strcat(playlistPath, mainPlaylistName);
-        mainPlaylist = malloc(sizeof(PlayList));
-        if (mainPlaylist == NULL)
+        specialPlaylist = malloc(sizeof(PlayList));
+        if (specialPlaylist == NULL)
         {
-                printf("Failed to allocate memory for mainPlaylist.\n");
+                printf("Failed to allocate memory for special playlist.\n");
                 exit(0);
         }
-        mainPlaylist->count = 0;
-        mainPlaylist->totalDuration = 0;
-        mainPlaylist->head = NULL;
-        mainPlaylist->tail = NULL;
-        readM3UFile(playlistPath, mainPlaylist);
+        specialPlaylist->count = 0;
+        specialPlaylist->head = NULL;
+        specialPlaylist->tail = NULL;
+        readM3UFile(playlistPath, specialPlaylist);
 }
 
-void saveMainPlaylist(const char *directory, bool isPlayingMain)
+void saveSpecialPlaylist(const char *directory, bool isPlayingMain)
 {
         char playlistPath[MAXPATHLEN];
         c_strcpy(playlistPath, sizeof(playlistPath), directory);
@@ -728,14 +626,14 @@ void saveMainPlaylist(const char *directory, bool isPlayingMain)
 
         if (isPlayingMain && playlist.count > 0)
                 writeM3UFile(playlistPath, &playlist);
-        else if (mainPlaylist != NULL && mainPlaylist->count > 0)
-                writeM3UFile(playlistPath, mainPlaylist);
+        else if (specialPlaylist != NULL && specialPlaylist->count > 0)
+                writeM3UFile(playlistPath, specialPlaylist);
 }
 
-void savePlaylist()
+void savePlaylist(const char *path)
 {
         char playlistPath[MAXPATHLEN];
-        c_strcpy(playlistPath, sizeof(playlistPath), settings.path);
+        c_strcpy(playlistPath, sizeof(playlistPath), path);
         if (playlistPath[strlen(playlistPath) - 1] != '/')
                 strcat(playlistPath, "/");
         strcat(playlistPath, playlistName);
@@ -785,20 +683,23 @@ Node *findTail(Node *head)
 }
 
 PlayList deepCopyPlayList(PlayList *originalList)
+{
+       PlayList newList = {NULL, NULL, 0, PTHREAD_MUTEX_INITIALIZER};
+
+       deepCopyPlayListOntoList(originalList, &newList);
+       return newList;
+}
+
+void deepCopyPlayListOntoList(PlayList *originalList, PlayList *newList)
 {
         if (originalList == NULL)
-        {
-                PlayList newList = {NULL, NULL, 0, 0.0};
-                return newList;
+        {               
+                return;
         }
 
-        PlayList newList;
-        newList.head = deepCopyNode(originalList->head);
-        newList.tail = findTail(newList.head);
-        newList.count = originalList->count;
-        newList.totalDuration = originalList->totalDuration;
-
-        return newList;
+        newList->head = deepCopyNode(originalList->head);
+        newList->tail = findTail(newList->head);
+        newList->count = originalList->count;
 }
 
 Node *findPathInPlaylist(char *path, PlayList *playlist)
@@ -833,25 +734,6 @@ Node *findLastPathInPlaylist(char *path, PlayList *playlist)
         return NULL;
 }
 
-Node *findSongInPlaylist(Node *currentSong, PlayList *playlist)
-{
-        Node *currentNode = playlist->head;
-
-        if (currentSong == NULL)
-                return NULL;
-
-        while (currentNode != NULL)
-        {
-                if (strcmp(currentNode->song.filePath, currentSong->song.filePath) == 0)
-                {
-                        return currentNode;
-                }
-
-                currentNode = currentNode->next;
-        }
-        return NULL;
-}
-
 int findNodeInList(PlayList *list, int id, Node **foundNode)
 {
         Node *node = list->head;
@@ -873,3 +755,31 @@ int findNodeInList(PlayList *list, int id, Node **foundNode)
 
         return -1;
 }
+
+void addSongToPlayList(PlayList *list, const char *filePath, int playlistMax) {
+    if (list->count >= playlistMax) return;
+
+    Node *newNode = NULL;
+    createNode(&newNode, filePath, list->count);
+    addToList(list, newNode);
+}
+
+void traverseFileSystemEntry(FileSystemEntry *entry, PlayList *list, int playlistMax) {
+    if (entry == NULL || list->count >= playlistMax) return;
+
+    if (entry->isDirectory == 0) {
+        addSongToPlayList(list, entry->fullPath, playlistMax);
+    }
+
+    if (entry->isDirectory == 1 && entry->children != NULL) {
+        traverseFileSystemEntry(entry->children, list, playlistMax);
+    }
+
+    if (entry->next != NULL) {
+        traverseFileSystemEntry(entry->next, list, playlistMax);
+    }
+}
+
+void createPlayListFromFileSystemEntry(FileSystemEntry *root, PlayList *list, int playlistMax) {
+    traverseFileSystemEntry(root, list, playlistMax);
+}
