@@ -1,24 +1,22 @@
+#include <FreeImage.h>
 #include <glib.h>
 #include <gio/gio.h>
 #include <sys/wait.h>
 #include <unistd.h>
-#include <FreeImage.h>
-#include "metadata.h"
-#include "cutils.h"
-#include "file.h"
 #include "cache.h"
 #include "chafafunc.h"
-#include "albumart.h"
-#include "soundgapless.h"
+#include "file.h"
+#include "sound.h"
 #include "soundcommon.h"
-
-#ifndef KEYVALUEPAIR_STRUCT
-#define KEYVALUEPAIR_STRUCT
+#include "utils.h"
 
 #ifndef MAXPATHLEN
 #define MAXPATHLEN 4096
 #endif
 
+#ifndef KEYVALUEPAIR_STRUCT
+#define KEYVALUEPAIR_STRUCT
+
 typedef struct
 {
         char *key;
@@ -49,22 +47,18 @@ typedef struct
         gchar *trackId;
         char filePath[MAXPATHLEN];
         char coverArtPath[MAXPATHLEN];
-        char pcmFilePath[MAXPATHLEN];
-        unsigned char *red;
-        unsigned char *green;
-        unsigned char *blue;
+        unsigned char red;
+        unsigned char green;
+        unsigned char blue;
         TagSettings *metadata;
         FIBITMAP *cover;
-        double *duration;
-        char *pcmFile;
-        long pcmFileSize;
+        double duration;
         bool hasErrors;
-        bool deleted;
 } SongData;
 
 #endif
 
+extern Cache *tempCache;
+
 SongData *loadSongData(char *filePath);
 void unloadSongData(SongData **songdata);
-double getDuration(const char *filepath);
-void stopFFmpeg(void);
