@@ -1,24 +1,31 @@
-#include <math.h>
+
+
 #include <float.h>
 #include <fftw3.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <math.h>
 #include <locale.h>
 #include <complex.h>
-#include "albumart.h"
-#include "soundgapless.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include "sound.h"
 #include "term.h"
-#include "write_ascii.h"
+
+#ifndef PIXELDATA_STRUCT
+#define PIXELDATA_STRUCT
+    typedef struct
+    {
+        unsigned char r;
+        unsigned char g;
+        unsigned char b;
+    } PixelData;
+#endif
 
 void initVisuals();
 
 void freeVisuals();
 
-void drawSpectrumVisualizer(int height, int width, PixelData c);
+void drawSpectrumVisualizer(int height, int width, PixelData c, int indentation, bool useProfileColors);
 
 PixelData increaseLuminosity(PixelData pixel, int amount);
 
-PixelData decreaseLuminosity(PixelData pixel, int amount);
-
 void printBlankSpaces(int numSpaces);
